/* <LICENSE>
License for the MULTOVL multiple genomic overlap tools

Copyright (c) 2007-2012, Dr Andras Aszodi, 
Campus Science Support Facilities GmbH (CSF),
Dr-Bohr-Gasse 3, A-1030 Vienna, Austria, Europe.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the Campus Science Support Facilities GmbH
      nor the names of its contributors may be used to endorse
      or promote products derived from this software without specific prior
      written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
</LICENSE> */
#ifndef MULTOVL_TEST_TEMPFILE_HEADER
#define MULTOVL_TEST_TEMPFILE_HEADER

// == Header tempfile.hh ==

// Little RAII utility to create/remove temporary files.
// \date 2011-12-02

// -- Boost headers --

// Use V3 of Boost::Filesystem, this is relevant between Boost versions 1.44 to 1.45
#define BOOST_FILESYSTEM_VERSION 3
#include "boost/filesystem.hpp"

// -- Standard headers --

#include <iostream>

// == Classes ==

class Tempfile
{
    public:
    Tempfile();
    ~Tempfile();
    
	// Returns the name of the temporary file.
	// Under Windows the pathnames are wide-char.
#ifdef _WIN32
	const wchar_t*
#else
    const char*
#endif
	name() const { return _path.native().c_str(); }
    
    // print the contents of the temp file to /outs/    
    std::ostream& print(std::ostream& outs = std::cout) const;
    
    private:
    boost::filesystem::path _path;
};

#endif  // MULTOVL_TEST_TEMPFILE_HEADER

